#ifndef _sim_H_
#define	_sim_H_

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/*********This file contains all the global variables and functions/subroutines prototypes of file qwsim.c**********/

/******************Structures*****************************/
typedef struct complex{
double re;
double im;
}complex;

typedef struct c_matrix{
	int rows;
	int cols;
	complex *t;
}c_matrix;

/*This macro is to be used with the structure c_matrix
 *m(A,i,j) accesses the line i and column j entry of matrix A
 * */
#define M(m,x,y) m.t[x+y*m.rows]


typedef struct state{
c_matrix direction;
}state;




/*******************************Global Variables/structures********************************/

//s1 and s2 are the overall state. s1 is the main structure and the operations on s1 are stored in s2. at the end of a step, s2 is copied to s1. This will avoid erasing values when performing shift operation.
state *s1, *s2;
//Number of steps for Quantum Walk
int steps;
//Dimension of the lattice
int N;
//Number of nodes on the lattice
int Nodes;
//Maximum number of positions of each particle
int Nodes1D;
//Number of edges per node (Only 4 is used)
int Edges;
//Counter for the number of steps performed
int t_counter;
//Number of fixed coins to be set
int fixed_coins;
//Fraction of link impurities
double index_brokenlink_imp;
//Range for the parameters theta, zeta, xi for each position of the line
double **range, **range_fixed;
//Number of links times index_brokenlink_imp
int N_imp;
//Number of fixed broken links
int fixed_broken_link;
//Fraction of coin impurities
double index_coin_imp;
//Number of positions times index_brokenlink_imp
int C_imp;
//Pointer for vector that stores the code for the coin operator at each position
int *pos_coin;
//Pointer for vector that stores the code for the coin operator at each position and per step alterations
int *pos_coin_altered;
//Choice of coin to apply on all positions of the line
int coin1;
//Choice for reflecting boundary
int ref;
//Choice for ploting output data with gnuplot
int gnu;
//Average position probability over time
double *P_av;
//Position probability
double *P;
//Pointer for vector that stores von Neumann entropy of coin state at each step
double *E_v;
//Pointer for vector that stores von Neumann entropy of position state at each step
double *E_v_p;
//Pointer for vector that stores Shannon entropy of coin state at each step
double *E_s;
//Pointer for vector that stores Shannon entropy of position state at each step
double *E_s_p;
//Pointer for vector storing mean value of particle's position at each step
double *mean_x;
//Variables used in plots for the first/second variables on the graph
double *x, *y;
//Pointer to vector that stores standard deviation of particle's position
double *xy_cov;
//Number of points for absortion
int dim_absorb;
//Pointer to matrix with information of absorbing point and direction
int **absorb;
// coin state of the system (trace over position)
c_matrix rho_c;
// position state of the system (trace over coin)
c_matrix rho_p;
//Matrix used as storage of the particular coin operator to be applied at each position and at each step
c_matrix random_coin;
//Matrix that stores the fixed random matrix for every position in one step
c_matrix randomt;
//Pointer to a matrix that stores the topology of the line
int **shift_cond;
//Pointer for matrix that stores topology of the lattice and per step alterations
int **shift_cond_altered;
//Sets the varying coin at each step, equal to every position 
int varying_coin_each_step;
/*Global variables initalization function*/
void initialize_global_variables();

/*pointer for matrix that stores in each step the probability measured specified position*/
double * measure_prob;
/*Complex number operations*/
complex c_sum(complex, complex);
complex c_mult(complex, complex);
complex c_div(complex, complex);
complex c_conj(complex);
double c_abs_sq(complex);
double c_angle(complex);


/*complex matrix operations*/
double random_0_1();
double vector_norm_sq(c_matrix);
void new_matrix(c_matrix * ,int ,int );
void print_matrix(c_matrix );
void erase_matrix(c_matrix *);
void matrix_mult( c_matrix, c_matrix, c_matrix);
void matrix_sum(c_matrix ,c_matrix ,c_matrix );
void c_times_matrix(c_matrix, complex , c_matrix);
void matrix_conj(c_matrix ,c_matrix );
void matrix_transpose(c_matrix, c_matrix);
void matrix_adjoint(c_matrix *, c_matrix );
void identity(c_matrix);
void copy_matrix(c_matrix, c_matrix);
c_matrix copy_sub_matrix(c_matrix ,int *, int *);
void random_matrix_1D();
void random_matrix_1D_2(int );
void random_matrix_1D_3(int );
void random_matrix_2D();
c_matrix random_rotation_2D();
int test_eigen_values(c_matrix, c_matrix);


/*Eigenvalues*/
void Householder_matrix(c_matrix, c_matrix);
c_matrix matrix_P(c_matrix , c_matrix );
c_matrix I_diag_B(c_matrix , int ); 
c_matrix QR_decomposition(c_matrix);
c_matrix QR_algorithm(c_matrix);



/**********************************Quantum Walk**************************************/
int biject(int, int);
int biject3D(int, int, int);
int biject1D(int );
void bijinv(int* ,int);
void bijinv3D(int*, int);
state * initialize_state();
void shift(state *,state*, int );
void shift2D(state *, state *, int );
void step(state *, state *, int );
void erase_state(state *);
void plot2D(char *,double *, double *, int);
void plot3D(char *, double *, int); 
void coin1D(state *, state *, int);
void position_trace_1D(state *, int);
void coin_trace_2D(state *,int );
void Hadamard1D();
void Hadamard2D();
void Fourier2D();
void Grover2D();
void absorbing_boundary(int ,int , int);

/*Broken links, and graph related functions*/
void initialize_shift_cond();
void initialize_shift_cond_altered();
void reflect1D();
void circle1D();
void moebius2D_i();
void moebius2D_j();
void broken_link2D(int , int , int);
void copy_shift_cond();
void link_impurities();
void initialize_position_coin();
void initialize_position_coin_altered();
void coin_impurities();
void reflect_coin_1D(c_matrix );
void initialize_absorb();
void permanent_broken_link1D(int , int);

/*Relevant Quantities*/
void mean_value_x(state *);
void covariance_x(state *);
complex entropy(c_matrix);
void sum_prop_pos(state *);
double prob_test(state *);
void P_av_actualization();

void plots();

void plot_on_line(state *s);

/*Quantum Walk Function*/
void qw_on_plane(state *s1, state *s2);



/*GNUplot functions*/
void gnu_plot2D(char *, double *, double *, int);
void gnu_plot3D(char *, double *, int);
void gnu_plots();

#endif
